#!/bin/sh /usr/share/centrifydc/perl/run

##############################################################################
#
# Copyright (C) 2014-2020 Centrify Corporation. All rights reserved.
#
# This script is used to modify netsvc.conf
#
# arguments:
#       please see sub usage
# return:
#       0 if succeed
#       1 if meet error
#
##############################################################################

use warnings;
use strict;
use lib "/usr/share/centrifydc/perl";
use lib "/usr/share/centrifydc/scripts";
use CentrifyDC::Include;
use Getopt::Long qw(GetOptions);

use autoedit_common_utils qw(loadSetting getValueList);
use autoedit_file_lock;
use CentrifyDC::GP::General qw(:debug :system);

sub usage
{
    warn << "END";
    This script is used to modify netsvc.conf
    usage
    --cdc_tag           the name of centrifydc lib
    --cfg_file          the full path of netsvc.conf
    --nss_xmaps         the maps to be updated
    --do_load_cdc_conf  set it if want to load the above settings
                        from centrifydc.conf
    --do_revert         set it if want to revert our changes
END
    exit 1;
}

use constant DEFAULT_CFG_FILE => "/etc/netsvc.conf";
use constant DEFAULT_NSS_XMAPS => [];
use constant DEFAULT_CDC_TAG => "nss_cdc";

my $cfgName = DEFAULT_CFG_FILE;
my $cfgHandler;
my $newName;
my $newHandler;

my $cdc_tag = DEFAULT_CDC_TAG;
my $nss_xmaps;
my $do_load_cdc_conf;
my $do_revert;

GetOptions( "cdc_tag=s" => \$cdc_tag,
            "cfg_file=s" => \$cfgName,
            "nss_xmaps=s" => \$nss_xmaps,
            "do_load_cdc_conf" => \$do_load_cdc_conf,
            "do_revert" => \$do_revert,
            ) or usage;

# need not to check all options when do revert
if (!$do_revert)
{
    if (defined $nss_xmaps)
    {
        $nss_xmaps = [split /[,\s]+/, $nss_xmaps];
    }
    else
    {
        if ($do_load_cdc_conf)
        {
            $nss_xmaps = getValueList("adclient.autoedit.nss.xmaps");
        }

        if (!defined $nss_xmaps)
        {
            $nss_xmaps = DEFAULT_NSS_XMAPS;
        }
    }

    # it only updates "hosts" in netsvc.conf
    # so, if "hosts" is not in $nss_xmaps, we exit without doing anything
    unless (grep /^hosts$/, @$nss_xmaps)
    {
        exit 0;
    }
}

my $flock = autoedit_file_lock::new("$cfgName");
if (!defined $flock)
{
    DEBUG_OUT("cannot lock file $cfgName");
    exit 1;
}

$cfgName = TraverseSymLink($cfgName);
if (!defined $cfgName)
{
    ERROR_OUT("meet error when check symlink $cfgName");
    exit 1;
}

my $cfgLines = ReadFile("$cfgName");
if (!defined $cfgLines)
{
    ERROR_OUT("cannot read $cfgName");
    exit 1;
}

if ($do_revert)
{
    if ($cfgLines !~ /\b$cdc_tag\b/)
    {
        exit 0;
    }

    # remove $cdc_tag we added
    $cfgLines =~ s/= $cdc_tag, /=/g;
    # remove other $cdc_tag if have
    $cfgLines =~ s/\b$cdc_tag\s*,?//g;
}
else
{
    # have been updated ?
    if ($cfgLines =~ /\b$cdc_tag\b/)
    {
        exit 0;
    }

    foreach my $map (@$nss_xmaps)
    {
        # If there is no setting specified in hosts stanza.
        # For example,
        # hosts =
        # or
        # hosts =     # comment...
        # Just skip it. Aix will use the setting specified in irs.conf file.
        $cfgLines =~ s/^(\s*$map\s*=)([ \t]*[^#\s]+)/$1 $cdc_tag, $2/m;
    }
}

my $backup = "$cfgName.pre_cdc";
if (!$do_revert)
{
    if (! CopyFile($cfgName, $backup))
    {
        ERROR_OUT("Cannot backup $cfgName");
        exit 1;
    }
}
else
{
    # install.sh and rpm/pkg scripts may override config files with
    # the backup files if found them exist.
    # To avoid affecting these code logic, we'd better delete
    # backup file after "do_revert"
    if (-e $backup)
    {
        unlink $backup;
    }
}

if (!WriteFile($cfgName, $cfgLines))
{
    ERROR_OUT("Cannot write $cfgName");
    exit 1;
}

exit 0;

