#!/bin/sh /usr/share/centrifydc/perl/run

##############################################################################
#
# Copyright (C) 2014-2020 Centrify Corporation. All rights reserved.
#
# This script is used to modify nscd.conf
#
# arguments:
#       please see sub usage
# return:
#       0 if succeed
#       1 if meet error
#
##############################################################################

use warnings;
use strict;
use lib "/usr/share/centrifydc/perl";
use lib "/usr/share/centrifydc/scripts";
use CentrifyDC::Include;
use Getopt::Long qw(GetOptions);

use autoedit_common_utils qw(loadSetting getValueList);
use autoedit_constants qw($CDC_Modified_Comment $CDC_Version);
use autoedit_file_lock;
use CentrifyDC::GP::General qw(:debug :system);

sub usage
{
    warn << "END";
    This script is used to modify nscd.conf
    usage:
    --cfg_file          the full path of nsswitch.conf
    --do_revert         set it if want to revert our changes
END
    exit 1;
}

use constant DEFAULT_CFG_FILE => "/etc/nscd.conf";

my $cfgName = DEFAULT_CFG_FILE;
my $cfgHandler;
my $newName;
my $newHandler;

my $do_load_cdc_conf;
my $do_revert;

my $cdc_long_comment = $CDC_Modified_Comment;
my $cdc_short_comment = "# modified by centrifydc";

GetOptions( "cfg_file=s" => \$cfgName,
            "do_revert" => \$do_revert,
            ) or usage;

my $flock = autoedit_file_lock::new("$cfgName");
if (!defined $flock)
{
    ERROR_OUT("cannot lock file $cfgName");
    exit 1;
}

$cfgName = TraverseSymLink($cfgName);
if (!defined $cfgName)
{
    ERROR_OUT("meet error when check symlink $cfgName");
    exit 1;
}

my $cfgLines = ReadFile("$cfgName");
if (!defined $cfgLines)
{
    ERROR_OUT("cannot read $cfgName");
    exit 1;
}

# Solaris11 use keyword "enable_cache", 
# while other OS use "enable-cache"
my $enable_cache = "enable-cache";
if ($cfgLines =~ /^[ \t]*enable_cache/)
{
    $enable_cache = "enable_cache"
}

if ($do_revert)
{
    $cfgLines =~ s/^$cdc_short_comment ?//mg;
    $cfgLines =~ s/^$cdc_long_comment.*\n.*\n//mg;
}
else
{
    if ($cfgLines =~ /^$cdc_long_comment/m)
    {
        # find the finger print of centrifydc
        exit 0;
    }

    $cfgLines =~ s/^[ \t]*$enable_cache[ \t]*passwd/$cdc_short_comment $&/mg;
    $cfgLines =~ s/^[ \t]*$enable_cache[ \t]*group/$cdc_short_comment $&/mg;

    if ($cfgLines !~ /\n$/)
    {
        $cfgLines .= "\n";
    }
    $cfgLines .= "$cdc_long_comment $CDC_Version\n"
                ."\t$enable_cache passwd no\n";
    $cfgLines .= "$cdc_long_comment $CDC_Version\n"
                ."\t$enable_cache group no\n";
}

my $backup = "$cfgName.pre_cdc";
if (!$do_revert)
{
    if (! CopyFile($cfgName, $backup))
    {
        ERROR_OUT("Cannot backup $cfgName");
        exit 1;
    }
}
else
{
    # install.sh and rpm/pkg scripts may override config files with
    # the backup files if found them exist.
    # To avoid affecting these code logic, we'd better delete
    # backup file after "do_revert"
    if (-e $backup)
    {
        unlink $backup;
    }
}

if (!WriteFile($cfgName, $cfgLines))
{
    ERROR_OUT("Cannot write $cfgName");
    exit 1;
}

exit 0;

