#!/bin/sh /usr/share/centrifydc/perl/run

##############################################################################
#
# Copyright (C) 2014-2020 Centrify Corporation. All rights reserved.
#
# This script is used to modify nsswitch.conf via svccfg
#
# arguments:
#       please see sub usage
# return:
#       0 if succeed
#       1 if meet error
#
##############################################################################

use warnings;
use strict;
use lib "/usr/share/centrifydc/perl";
use lib "/usr/share/centrifydc/scripts";
use CentrifyDC::Include;
use Getopt::Long qw(GetOptions);

use autoedit_common_utils qw(loadSetting getValueList flushNameService);
use CentrifyDC::GP::General qw(:debug :system);

sub usage
{
    warn << "END";
    This script is used to modify nsswitch.conf via svccfg
    usage:
    --lib_name          the name of centrifydc or centrifyda lib
    --nss_xmaps         the maps to be updated
    --xxx_reaction      the reaction of map xxx
    --do_load_cdc_conf  set it if want to load the above settings
                        from centrifydc.conf
    --do_revert         set it if want to revert our changes
END
    exit 1;
}

my @PRIORITY = ("centrifyda", "centrifydc");
use constant DEFAULT_NSS_XMAPS => [];
use constant DEFAULT_CDC_TAG => "centrifydc";

my $lib_name = DEFAULT_CDC_TAG;
my $nss_xmaps;
my $do_load_cdc_conf;
my $do_revert;

# @nssKeys don't include "passwd", "group"
# because we have no reactions for them
my @nssKeys = ( "netgroup", "protocols", "rpc", "hosts", "ethers", 
                "aliases", "services", "prof_attr", "auth_attr", 
                "bootparams", "netmasks", "printers", "project", "networks");

# a hash for svccfg, which use different name to nsswitch.conf
# Do not process shadow, Solaris have no shadow
# Do not process ipnodes, it is configured together with hosts
my %svcName = ( "default" => "default",
                "passwd" => "password",
                "group" => "group",
                "hosts" => "host",
                "netgroup" => "netgroup",
                "protocols" => "protocol",
                "rpc" => "rpc",
                "ethers" => "ether",
                "networks" => "network",
                "aliases" => "alias",
                "services" => "service",
                "prof_attr" => "prof_attr",
                "auth_attr" => "auth_attr",
                "bootparams" => "bootparam",
                "netmasks" => "netmask",
                "printers" => "printer",
                "project" => "project",
                );

my %nss_reactions = ();
my %nss_reaction_opts = ();
foreach (@nssKeys)
{
    $nss_reactions{$_} = undef;
    $nss_reaction_opts{"$_"."_reaction=s"} = \$nss_reactions{$_};
}

GetOptions( "lib_name=s" => \$lib_name,
            "nss_xmaps=s" => \$nss_xmaps,
            "do_load_cdc_conf" => \$do_load_cdc_conf,
            "do_revert" => \$do_revert,
            %nss_reaction_opts
            ) or usage;

# need not to check all options when do revert
if (!$do_revert)
{
    if (defined $nss_xmaps)
    {
        $nss_xmaps = [split /[,\s]+/, $nss_xmaps];
    }
    else
    {
        if ($do_load_cdc_conf)
        {
            $nss_xmaps = getValueList("adclient.autoedit.nss.xmaps");
        }

        if (!defined $nss_xmaps)
        {
            $nss_xmaps = DEFAULT_NSS_XMAPS;
        }
    }

    # get reaction for nss maps
    foreach my $key (@nssKeys)
    {
        if (!defined $nss_reactions{$key})
        {
            if ($do_load_cdc_conf)
            {
                # Get the user specified nss db reaction using key:
                # adclient.autoedit.nss.<name>.reaction: <value>
                # Ensure the reaction is enclosed with [ ]. 
                my $array_ref = getValueList("adclient.autoedit.nss.$key.reaction");
                if ($array_ref)
                {
                    my $reaction = $array_ref->[0];
                    if ($reaction && $reaction =~ /^\[[^\[\]]+\]$/)
                    {
                        $nss_reactions{$key} = $reaction;
                    }
                }
            }
        }
    }

    # the default reaction for "netgroup"
    if (!defined $nss_reactions{"netgroup"})
    {
        $nss_reactions{"netgroup"} = "[NOTFOUND=return]";
    }
}

# the priority for the lib_name we support
# it may has multiple different lib_names in one line
# we need to sort them by priority
my %priority = ();
if (grep /^$lib_name$/, @PRIORITY)
{
    my $count = 0;
    foreach my $item (@PRIORITY)
    {
        $priority{$item} = $count;
        $count++;
    }
}
else
{
    # it is an unknown $lib_name
    # place it at top
    $priority{$lib_name} = 0;
}

# determine which maps need to update
my %keysToCheck = ();
if ($lib_name eq "centrifyda")
{
    %keysToCheck = ('passwd' => 1);
}
else
{
    %keysToCheck = ('passwd' => 1,
                    'group' => 1);
}

if ($do_revert)
{
    foreach my $key (@nssKeys)
    {
        $keysToCheck{$key} = 1;
    }
}
else
{
    foreach my $key (@$nss_xmaps)
    {
        if (grep /^$key$/, @nssKeys)
        {
            $keysToCheck{$key} = 1;
        }
    }
}

my $svccfg = "/usr/sbin/svccfg";
my $default = "";

# get default value first, then update switch svc
foreach ("default", keys %keysToCheck)
{
    my $key = $svcName{$_};
    # get config/default
    my $ret = `$svccfg -s name-service/switch listprop config/$key 2>&1`;
    chomp($ret);

    # get value
    my $value;
    if ($ret eq "")
    {
        $value = $default;
    }
    elsif ($ret =~ /\s*config\/\w+\s+astring\s+(.*)/)
    {
        # e.g. config/default astring     files
        $value = $1;
        $value =~ s/"//g;
    }
    else
    {
        DEBUG_OUT("got un-expected return from svccfg : $ret");
        exit 1;
    }

    if ($key eq "default")
    {
        $default = $value;
        next;
    }

    # update value
    if ($do_revert)
    {
        if ($value =~ /\b$lib_name\b/)
        {
            # remove $lib_name and its reaction (if have)
            $value =~ s/\s*$lib_name(\s*\[.*\])?\s*/ /g
        }
        else
        {
            # have nothing to change
            next;
        }
    }
    else
    {
        if ($value =~ /\b$lib_name\b/)
        {
            # have been using $lib_name
            next;
        }
        else
        {
            my $reaction = defined $nss_reactions{$_} ? " $nss_reactions{$_}" : "";
            #$value = "$lib_name $reaction".$value;
            # prepare our code to insert
            my @ours = ("") x (scalar (keys %priority));
            foreach my $item (keys %priority)
            {
                if ($value =~ s/\s*($item(\s*\[.*\])?)\s*/ /)
                {
                    $ours[$priority{$item}] = " $1";
                }
            }
            $ours[$priority{$lib_name}] = " $lib_name$reaction";
            my $ours = join "", @ours;
            
            $value = $ours . " " . $value;
        }
    }

    if ($value eq $default)
    {
        # delete it to use config/default
        $ret = `$svccfg -s name-service/switch delprop config/$key 2>&1`;
    }
    else
    {
        # set it
        $ret = `$svccfg -s name-service/switch setprop "config/$key=\\"$value\\"" 2>&1`;
    }

    chomp($ret);
    if ($ret)
    {
        DEBUG_OUT("got un-expected return from svccfg : $ret");
        exit 1;
    }

    $ret = system("/usr/sbin/svcadm refresh name-service/switch");
    if ($ret)
    {
        DEBUG_OUT("got un-expected return from svcadm : $ret");
        exit 1;
    }

    # flush name service cache
    flushNameService();
}

exit 0;

